import { Component, OnInit } from '@angular/core';
import { AgentsService } from '../../services/agent.service';
import { ActivatedRoute, Router } from '@angular/router';
import { Agent } from '../../interfaces/agent.interface';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Location } from '@angular/common';

@Component({
  selector: 'app-agent-add',
  templateUrl: './agent-add.component.html',
  styleUrls: ['./agent-add.component.css']
})
export class AgentAddComponent implements OnInit {

  public myForm!: FormGroup;
  private isNew: boolean = false;

  constructor(
    private _supervisorService:
    AgentsService, private _activatedRoute:
    ActivatedRoute, private _router: Router,
    private fb: FormBuilder,
    private location: Location) {}

  ngOnInit(): void {
    this._activatedRoute.params.subscribe(param => {
      if (param['id']) {
        const id: number = param['id'];
        const agent = this._supervisorService.getAgentById(id);
        if (agent) {
          this.buildForm(agent);
        } else {
          this._router.navigate(['/supervisors']);
        }
      } else {
        this.buildForm();
      }
    })
  }

  isValidField(field: string): boolean | undefined {
    const fieldForm = this.myForm.get(field);
    return fieldForm?.invalid && fieldForm?.touched;
  }

  getErrorMessage(field: string): string | null {
    if (!this.myForm.controls[field]) return null;

    const errors = this.myForm.controls[field].errors || {};

    console.log(errors);


    for (const key of Object.keys(errors)) {
      switch (key) {
        case 'required':
          return 'Este campo es requerido';
        case 'minlength':
          return `Este campo debe tener al menos ${errors[key].requiredLength} caracteres`;
        case 'maxlength':
          return `Este campo debe tener como máximo ${errors[key].requiredLength} caracteres`;
        case 'email':
          return 'El email no es válido';
      }
    }

    return null;
  }

  dataLengths = {
    name: {
      min: 3,
      max: 20
    },
    lastname: {
      min: 3,
      max: 30
    },
    detail: {
      min: 3,
      max: 100
    },
    email: {
      min: 3,
      max: 30
    },
    password: {
      min: 3,
      max: 30
    }
  };

  buildForm(agent?: Agent):void {
    if (agent) {
      this.myForm = this.fb.group({
        name: [agent.name, [Validators.required, Validators.minLength(3), Validators.maxLength(this.dataLengths['name'].max)]],
        lastname: [agent.lastname, [Validators.required, Validators.minLength(3), Validators.maxLength(this.dataLengths['lastname'].max)]],
        detail: [agent.detail, [Validators.required, Validators.minLength(3), Validators.maxLength(this.dataLengths['detail'].max)]],
        email: [agent.email, [Validators.required, Validators.minLength(3), Validators.maxLength(this.dataLengths['email'].max)]],
        password: [agent.password, [Validators.required, Validators.minLength(3), Validators.maxLength(this.dataLengths['password'].max)]]
      })

    } else {
      this.myForm = this.fb.group({
        name: ['', [Validators.required, Validators.minLength(3), Validators.maxLength(20)]],
        lastname: ['', [Validators.required, Validators.minLength(3), Validators.maxLength(30)]],
        detail: ['', [Validators.required, Validators.minLength(3), Validators.maxLength(100)]],
        email: ['', [Validators.required, Validators.minLength(3), Validators.maxLength(30)]],
        password: ['', [Validators.required, Validators.minLength(3), Validators.maxLength(30)]]
      });
      this.isNew = true;
      console.log(this.myForm);
    }
  }

  onSave() {
    this.myForm.markAllAsTouched();
    if (this.myForm.valid) {
      if (this.isNew) {
        this._supervisorService.saveAgent(this.myForm.value);
      } else {
        const id: number = this._activatedRoute.snapshot.params['id'];
        this._supervisorService.editAgent(id, this.myForm.value);
      }
      this.myForm.reset({name: '', lastname: '', detail: '', email: '', password: ''});
      this.location.back();
    }
  }
}
